from pathlib import Path
import os

import yaml

user_home = os.path.expanduser('~')


def __get_input__(t: str):
    return input(t)


def initial_prompt():
    should_upload_log = False

    warning = '''
        VerinFast™ would like to collect information about
        this scan. Do you give consent to upload diagnostic
        information to our sponsor, StartupOS at:
        https://beta.startupos.dev/
    '''

    storage = f'''
        VerinFast™ stores user preferences and configurations
        in your home directory. To do that we create a hidden
        folder (.verinfast) inside the directory {user_home}.

        We never store keys, passwords or confidential information
        there. We do store consents so we don't prompt on every
        run, hashes for more efficient scanning, and other
        preferences.
    '''

    f_path = (
        Path(os.path.expanduser('~/.verinfast/'))
        .joinpath('preferences.yaml')
    )
    conf = {}
    if f_path.exists():
        with open(f_path, "r") as f:
            conf = yaml.safe_load(f)

    repeat_prompt = "(Y)/n\n"
    if "upload_permission" not in conf:
        print(warning)
        resp = __get_input__(repeat_prompt)
        print()
        if resp:
            resp_char = resp[0]
        else:
            resp_char = 'y'
        while resp_char.lower() not in ['y', 'n']:
            resp = __get_input__(repeat_prompt)
            if resp:
                resp_char = resp[0]
            else:
                resp_char = 'y'
        if resp_char.lower() == 'y':
            conf["upload_permission"] = True
            should_upload_log = True
        else:
            conf["upload_permission"] = False
    else:
        should_upload_log = conf["upload_permission"]

    if "show_warning" not in conf or conf["show_warning"]:
        print(storage)
        conf["show_warning"] = False

    os.makedirs(os.path.expanduser('~/.verinfast'), exist_ok=True)

    with open(f_path, "w") as f:
        f.write(yaml.dump(conf))

    return should_upload_log


def save_path():
    return os.path.expanduser('~/.verinfast/')


if __name__ == "__main__":
    print(initial_prompt())



import json
from pathlib import Path

from pygments_tsx.tsx import patch_pygments

from verinfast.dependencies.walk import walk
from verinfast.dependencies.walkers.classes import Entry

file_path = Path(__file__)
test_folder = file_path.parent

patch_pygments()


def test_walk():
    output_path = walk(path=test_folder, output_file="./dependencies.json")
    with open(output_path) as output_file:
        output = json.load(output_file)
        assert len(output) >= 1
        first_dep = output[0]
        assert first_dep['name'] == 'simple-test-package'
    return True


def test_entity():
    output_path = walk(path="./tests/", output_file="./dependencies.json")
    with open(output_path) as output_file:
        output = json.load(output_file)
        assert len(output) >= 1
        first_dep = output[0]
        e = Entry(**first_dep)
        assert e.license == "ISC"

    return True



import json
import os
import subprocess

from verinfast.utils.utils import DebugLog
debugLog = DebugLog(os.getcwd())


def runAws(targeted_account, start, end, path_to_output):
    results = subprocess.run(
        'aws configure list-profiles',
        shell=True,
        stdout=subprocess.PIPE
    )

    text = results.stdout.decode()
    profiles = []
    available_accounts = []
    for line in text.splitlines():
        profiles.append(line)
        cmd = f'aws sts get-caller-identity --profile={line}'
        results = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE)
        text2 = results.stdout.decode()
        identity = json.loads(text2)
        account = identity["Account"]
        available_accounts.append(account)
        if str(account) == str(targeted_account):
            cmd = f'''
                aws ce get-cost-and-usage \
                --time-period Start={start},End={end} \
                --granularity=DAILY \
                --metrics "BlendedCost" \
                --group-by Type=DIMENSION,Key=SERVICE \
                --profile={line} | cat
            '''

            try:
                results = subprocess.run(
                    cmd,
                    shell=True,
                    stdout=subprocess.PIPE
                )

            except subprocess.CalledProcessError:
                raise Exception("Error getting aws cli data.")

            text = results.stdout.decode()
            obj = json.loads(text)
            results_by_time = obj["ResultsByTime"]
            charges = []
            for charge in results_by_time:
                if charge["Groups"]:
                    for group in charge["Groups"]:
                        newCharge = {
                            "Date": charge["TimePeriod"]["Start"],
                            "Group": group["Keys"][0],
                            "Cost": group["Metrics"]["BlendedCost"]["Amount"],
                            "Currency": group["Metrics"]["BlendedCost"]["Unit"]
                        }
                        charges.append(newCharge)
            upload = {
                "metadata": {
                    "provider": "aws",
                    "account": str(targeted_account)
                },
                "data": charges
            }
            aws_output_file = os.path.join(
                path_to_output,
                f'aws-cost-{targeted_account}.json'
            )

            with open(aws_output_file, 'w') as outfile:
                outfile.write(json.dumps(upload, indent=4))
            return aws_output_file
    debugLog.log(msg=profiles, tag="AWS Profiles")
    debugLog.log(msg=available_accounts, tag="AWS Available Accounts")



from datetime import datetime, timedelta

import json
import os

import boto3

import verinfast.cloud.aws.regions as r

from verinfast.utils.utils import DebugLog
debugLog = DebugLog(os.getcwd())

regions = r.regions


def getBlocks(sub_id: str, path_to_output: str = "./"):
    session = boto3.Session()
    profiles = session.available_profiles
    right_session = None
    for profile in profiles:
        s2 = boto3.Session(profile_name=profile)
        sts = s2.client('sts')
        id = sts.get_caller_identity()
        if int(id['Account']) == int(sub_id):
            right_session = s2
            break
    if right_session is None:
        return None
    # for region in regions:
    #     try:
    s3 = right_session.client('s3', region_name=regions[0])
    response = s3.list_buckets()
    known_buckets = {}
    for bucket in response['Buckets']:
        bucket_name = bucket["Name"]
        resp = s3.get_bucket_location(Bucket=bucket_name)

        permissions = []
        try:
            policy_status_resp = s3.get_bucket_policy_status(Bucket=bucket_name)
            public = policy_status_resp["PolicyStatus"]["IsPublic"]

            if "Policy" in policy_status_resp:
                p_string = policy_status_resp["Policy"]
                p_dict = json.loads(p_string)
                statements = p_dict["Statement"]
                # print(statements)
                permissions = [json.dumps(s) for s in statements]
                # print(s2)
        except s3.exceptions.from_code('NoSuchBucketPolicy'):
            debugLog.log(msg=bucket_name, tag="No Bucket Policy for bucket")

        versioning = None
        try:
            versioning_response = s3.get_bucket_versioning(Bucket=bucket_name)
            if "Status" in versioning_response:
                versioning = versioning_response["Status"]

        except s3.exceptions.from_code('NoSuchBucketStatus'):
            debugLog.log(msg=bucket_name, tag="No Bucket Status for bucket")

        region = resp['LocationConstraint']
        # print(region)
        if region:
            # print('Have region')
            cloudwatch = right_session.client('cloudwatch', region_name=region)
        else:
            cloudwatch = right_session.client(
                'cloudwatch',
                region_name='us-east-1'
            )

        response = cloudwatch.get_metric_statistics(
            Namespace="AWS/S3",
            MetricName="BucketSizeBytes",
            Dimensions=[
                {
                    u"Name": u"BucketName",
                    u"Value": bucket_name
                },
                {
                    u'Name': 'StorageType',
                    u'Value': 'StandardStorage'
                }
            ],
            StartTime=datetime.now() - timedelta(days=2),
            EndTime=datetime.now(),
            Period=3600,
            Statistics=['Average'],
            Unit="Bytes"
        )
        if response['Datapoints']:
            bucket_size_bytes = response['Datapoints'][-1]['Average']
            known_buckets[bucket_name] = {
                "name": bucket_name,
                "size": int(bucket_size_bytes),
                "retention": versioning,
                "public": public,
                "permissions": permissions
            }
        else:
            pass
            # print(response)
        # except:
        #     pass

    my_buckets = list(known_buckets.values())
    upload = {
                "metadata": {
                    "provider": "aws",
                    "account": str(sub_id)
                },
                "data": my_buckets
            }
    aws_output_file = os.path.join(
        path_to_output,
        f'aws-storage-{sub_id}.json'
    )

    with open(aws_output_file, 'w') as outfile:
        outfile.write(json.dumps(upload, indent=4))
    return aws_output_file

# Test Code
# getBlocks(sub_id='436708548746')
